;; This example shows the correct method to read the keyboard and
;; joysticks on the CPC, CPC+ and KC Compact.
;;
;; This source is compatible with the CPC+.
;;
;; The following is assumed before executing of this algorithm:
;; - I/O port A of the PSG is set to input,
;; - PPI Port A is set to output
;;

;;--------------------------------------------------------------------------------
;; example code showing how to use read_matrix.
;PrintChar equ &bb5a
;org &4000
;nolist

;.main_loop
;;; wait for vsync
;ld b,&f5
;.v1 in a,(c)
;rra
;jr nc,v1

;call read_matrix

;; NOTE: Consult the 'matrix' table in the
;; document 'Scanning the Keyboard & Joysticks' to find the keyboard line
;; and bit for the keys you want to check.

;; test the 'return' key has been pressed.
;; (line 2, bit 2).
;ld a,(matrix_buffer+2)
;bit 2,a
;jr z,not_pressed

;; return key pressed
;ld a,"a"
;call PrintChar
;.not_pressed

;jp main_loop



;; This buffer has one byte per keyboard line. 
;; Each byte defines a single keyboard line, which defines 
;; the state for up to 8 keys.
;;
;; A bit in a byte will be '1' if the corresponding key 
;; is pressed, '0' if the key is not pressed.

matrix_buffer:
  defs 10  ;map with 10*8 = 80 key status bits (bit=0 key is pressed)

read_matrix:
  ; WAIT FLYBACK
  ld b,&f5
  v1b2:
  in a,(c)
  rra
  jr nc,v1b2

keyscan di              ;1 ##%%## C P C   VERSION ##%%##
        ld hl,matrix_buffer    ;3
        ld bc,#f782     ;3
        out (c),c       ;4
        ld bc,#f40e     ;3
        ld e,b          ;1
        out (c),c       ;4
        ld bc,#f6c0     ;3
        ld d,b          ;1
        out (c),c       ;4
        ld c,0          ;2
        out (c),c       ;4
        ld bc,#f792     ;3
        out (c),c       ;4
        ld a,#40        ;2
        ld c,#4a        ;2 44
loopa   ld b,d          ;1
        out (c),a       ;4 select line
        ld b,e          ;1
        ini             ;5 read bits and write into KEYMAP
        inc a           ;1
        cp c            ;1
        jr c,loopa      ;2/3 9*16+1*15=159
        ld bc,#f782     ;3
        out (c),c       ;4
        ei              ;1 8 =211 microseconds
        ret
; INPUTS
; A = LINE TO READ &40-49
read_matrix_oneline:
keyscan2 di              ;1 ##%%## C P C   VERSION ##%%##
;        ld hl,matrix_buffer    ;3
         LD D,0
         LD BC,#F782 ; PPI port A out /C out 
         OUT (C),C 
         LD BC,#F40E ; Select Ay reg 14 on ppi port A 
         OUT (C),C 
         LD BC,#F6C0 ; This value is an AY index (R14) 
         OUT (C),C 
         OUT (C),D ; Validate!! out (c),0
         LD BC,#F792 ; PPI port A in/C out 
         OUT (C),C   
         DEC B 
         OUT (C),A ; Send KbdLine on reg 14 AY through ppi port A
         LD B,#F4 ; Read ppi port A  
         IN A,(C) ; e.g. AY R14 (AY port A) 
         LD BC,#F782 ; PPI port A out / C out   
         OUT (C),C 
         DEC B ; Reset PPI Write 
         OUT (C),D ; out (c),0
         ei              ;1 8 =211 microseconds
ret


waitoption_1_2_3b:
  ; LOOP UNTIL WE GET EITHER Y OR N PRESSED
  call read_matrix

  ; TEST 1
  ld a,(matrix_buffer+8)
  bit 0,a     
  jr nz,not_pressed_1
  ld a,"1"                         ; WE HAVE PRESSED 1
  ret
  not_pressed_1:
  ; TEST 2
  bit 1,a     
  jr nz,not_pressed_2
  ld a,"2"                         ; WE HAVE PRESSED 2
  ret
  not_pressed_2:
  ; TEST 3
  ld a,(matrix_buffer+7)
  bit 1,a     
  jr nz,waitoption_1_2_3b
  ld a,"3"                         ; WE HAVE PRESSED 3
  ret

waitoption_1_2_3_4_5b:
  ; LOOP UNTIL WE GET EITHER Y OR N PRESSED
  call read_matrix

  ; TEST 1
  ld a,(matrix_buffer+8)
  bit 0,a     
  jr nz,not_pressed__1
  ld a,"1"                         ; WE HAVE PRESSED 1
  ret
  not_pressed__1:
  ; TEST 2
  bit 1,a     
  jr nz,not_pressed__2
  ld a,"2"                         ; WE HAVE PRESSED 2
  ret
  not_pressed__2:
  ; TEST 3
  ld a,(matrix_buffer+7)
  bit 1,a     
  jr nz,not_pressed__3
  ld a,"3"                         ; WE HAVE PRESSED 3
  ret
  not_pressed__3:
  bit 0,a     
  jr nz,not_pressed__4
  ld a,"4"                         ; WE HAVE PRESSED 3
  ret
  not_pressed__4:
  ld a,(matrix_buffer+6)
  bit 1,a     
  jr nz,waitoption_1_2_3_4_5b
  ld a,"5"                         ; WE HAVE PRESSED 3
  ret


; LOOP UNTIL WE GET EITHER Y OR N PRESSED
waitoption_yes_no:
  ;; wait for vsync
  call waitvsync
  ld a,&45 ; KEYBOARD LINES &40-49
  call read_matrix_oneline

  ; TEST Y
  bit 3,a     
  jr nz,not_pressed_y
  ld a,"y"                         ; WE HAVE PRESSED Y
  ret
  not_pressed_y:

  ; TEST N
  bit 6,a     
  jr nz,waitoption_yes_no
  ld a,"n"                        ; WE HAVE PRESSED N
ret

waitvsync:
  ld b,&f5
  v1a3:
  in a,(c)
  rra
  jr nc,v1a3
ret

; LOOP UNTIL WE GET SPACEBAR PRESSED
waitoption_spacebar:
  ;; wait for vsync
  call waitvsync
  ld a,&45 ; KEYBOARD LINES &40-49
  call read_matrix_oneline
  ; TEST SPACE
  bit 7,a     
  jr nz,waitoption_spacebar
  ret

; WAIT UNTIL NO KEYS OR JOYSTICKS ARE BEING PRESSED
checknokeyspressed:
  push af            ; STORE KEYPRESS
  checkallkeysloop:
  call read_matrix
  ld hl,matrix_buffer
  ld b,10
  checkkeyloop:
    ld a,(hl)
    cp &FF
    jr nz,checkallkeysloop
    inc hl
  djnz checkkeyloop
  pop af             ; RESTORE KEYPRESS
ret

check_keys:
  call read_matrix

  ;; NOTE: Consult the 'matrix' table in the
  ;; document 'Scanning the Keyboard & Joysticks' to find the keyboard line
  ;; and bit for the keys you want to check.

  ;; test the 'return' key has been pressed.
  ;; (line 2, bit 2).

  ; USE B TO STORE WHICH DIRECTION WE ARE MOVING IN.
  ; 0 = NONE, 1 = NORTH, 2 = NORTH EAST, ETC

  ld a,(matrix_buffer+9)
  ld b,0 ; PRESSED NO DIRECTION
  ld c,0 ; PRESSED NO OTHER KEY
  bit 2,a
  jr nz,not_pressed_left 
    ld b,7                           ; WE HAVE PRESSED LEFT
    bit 0,a                        
    jr nz,not_pressed_left_up
    ld b,8                         ; WE HAVE PRESSED LEFT UP
    jr finishcheckjoy
    not_pressed_left_up:

    bit 1,a
    jr nz,not_pressed_left_down
    ld b,6                         ; WE HAVE PRESSED LEFT DOWN
    jr finishcheckjoy
    not_pressed_left_down:
  not_pressed_left:

  bit 3,a
  jr nz,not_pressed_right
    ld b,3                         ; WE HAVE PRESSED RIGHT
    bit 0,a     
    jr nz,not_pressed_right_up
    ld b,2                         ; WE HAVE PRESSED RIGHT UP
    jr finishcheckjoy
    not_pressed_right_up:

    bit 1,a                
    jr nz,not_pressed_right_down:
    ld b,4                         ; WE HAVE PRESSED RIGHT DOWN
    jr finishcheckjoy
    not_pressed_right_down:
  not_pressed_right:

  bit 0,a     
  jr nz,not_pressed_up
  ld b,1                         ; WE HAVE PRESSED UP
  jr finishcheckjoy
  not_pressed_up:

  bit 1,a     
  jr nz,not_pressed_down
  ld b,5                         ; WE HAVE PRESSED DOWN
  jr finishcheckjoy
  not_pressed_down:

  ; PRESSED M
  ld a,(matrix_buffer+4)
  bit 6,a                       
  jr nz,not_pressed_m
  ld c,1                         ; RECORD WE PRESSED M
  not_pressed_m:

  ; PRESSED H
  ld a,(matrix_buffer+5)
  bit 4,a                       
  jr nz,not_pressed_h
  ld c,6                         ; RECORD WE PRESSED H
  not_pressed_h:

  ; PRESSED Q
  ld a,(matrix_buffer+8)
  bit 3,a                       
  jr nz,not_pressed_q
  ld c,5                         ; RECORD WE PRESSED Q
  not_pressed_q:

  ; INVENTORY VIEWER

  ; LEFT ARROW
  ;ld a,(matrix_buffer+1)
  ;bit 0,a
  ;jr nz,not_pressed_leftarrow
  ;ld c,2                         ; RECORD WE PRESSED LEFT ARROW
  ;not_pressed_leftarrow:

  ;; RIGHT ARROW
  ;ld a,(matrix_buffer+0)
  ;bit 1,a       
  ;jr nz,not_pressed_rightarrow
  ;ld c,3                         ; RECORD WE PRESSED RIGHT ARROW
  ;not_pressed_rightarrow:

  ; PRESSED SPACE
  ld a,(matrix_buffer+5)
  bit 7,a                        
  jr nz,not_pressed_space
  ld a,(key_pressed)             ; CHECK TO SEE IF USER IS HOLDING SPACE DOWN
  bit 0,a                        ; IF SO WE DON'T WANT TO FIRE LOTS OF ARROWS
  jr nz,finishcheckjoy
  set 0,a
  ld (key_pressed),a             ; SET KEY PRESSED SPACE
  ld c,4                         ; RECORD WE PRESSED SPACE
  jr finishcheckjoy
  not_pressed_space:
  res 0,a
  ld (key_pressed),a           ; CLEAR PRESSED SPACE 

  finishcheckjoy:
  ; LOAD DIRECTION INTO PLAYER DATA, EVEN IF WE HAVEN'T PRESSED JOYSTICK
  ; MUST BE 0 IF NOT MOVING
  ld a,c
  or a;cp 0
  jr z,checkifpresseddirection
  jr checkkeyfunction
  checkifpresseddirection:
  ld a,b
  or a;cp 0
  ret z

  ; ONLY UPDATE IF WE PRESSED A KEY
  ; USE LOOKUP 
  ld ix,playerspritedata
  ld (ix+8),a ; SET PLAYER DIRECTION
  jp movespritep
  ;jumpback2:
  ;jp updateplayersprite

; INPUT 
; A = FUNCTION TO CALL
checkkeyfunction:
  ld hl,functionskeypresses
  jp vectortablelaunchcode

; KEY PRESSES OTHER THAN JOYSTICK
functionskeypresses:
  defw 0
  defw togglemap            ; 1 M
  defw 0;scrollinventoryleft  ; 2 LEFT CURSOR
  defw 0;scrollinventoryright ; 3 RIGHT CURSOR
  defw firearrow            ; 4 SPACE
  defw showquests           ; 5 Q
  defw showinventory        ; 6 H
