; INPUTS
; B = NUMBER OF POINTS TO INSERT INTO BLOCK OF 10X10 (256 ALLOCATED TO MAKE IT SIMPLE)
; C = BYTE TO SET
loopit:
   call rand      ; GET RAND OF 256
   ld l,a
   ld a,(hl)      ; DO NOT INSERT INTO MAP SQUARE IF IT IS ALREADY OCCUPIED
   or a;cp 0
   jr nz,skipbyteinsert
   ld (hl),c      ; INSERT BYTE INTO MAP
   skipbyteinsert:
   djnz loopit
ret

genlocalgamemap:
  ; CLEAR OLD DATA
  call cleargamemap
  call buildthesea

  ; MAKE SURE WE ARE CALCULATING BASED ON CURRENT BLOCK ID
  call resetpseudorandomblock 
  ; INSERT TILES THAT CAN BE BROKEN UP BY TOWNS
  call getranlocalmappos
  call doinsertforest

  ; INSERT STATIC MAP DATA
  call checkstaticmapdef

  ; GENERATE RANDOM SCENERY, HILLS, FORESTS, ISLANDS
  ; THESE ARE LARGE TILES THAT CAN'T BE BROKEN UP BY TOWNS
  call getranlocalmappos
  call doinsertisland
  call getranlocalmappos
  call doinsertmount
  call getranlocalmappos
  call doinsertlake
  call getranlocalmappos
  call doinsertroads

  ; GET RANDOM TOWN TYPE TO BUILD BASED ON ZONE
  ld hl,randomtownbyzone
  ld a,(currzonedata)
  ld b,0
  ld c,a
  add hl,bc
  ld c,(hl)  ; LOAD RANDOM TOWN IDENTIFIER INTO C
  
  ; INSERT RANDOM TOWN INTO MAP IF NEEDED
  ld hl,gamemap2
  ld b,30                        ; MAX NUMBER OF RANDOM POINTS TO GENERATE
  call loopit  
ret

;  defb 9,&FF     ; 0 zone_empty,PLAINS - GREEN
;  defb 9,&FF     ; 1 zone_forest,FOREST - GREEN
;  defb 9,&FF     ; 2 zone_indian,INDIAN - GREEN
;  defb 9,&FF     ; 3 zone_knights,KNIGHTS - GREEN
;  defb 15,&FC    ; 4 zone_western,WESTERN - ORANGE
;  defb 9,&FF     ; 5 zone_indian,TROPICAL - GREEN
;  defb 10,&0F    ; 6 zone_tropical,SEA - MAUVE
;  defb 10,&0F    ; 7 zone_sea,WINTER - BLUE
;  defb 9,&FF     ; 8 zone_winter,AUTUMN - GREEN
;  defb 9,&FF     ; 9 zone_autumn,AUTUMN - GREEN

mapdef_island:
  defb 3,3 ; WIDTH / HEIGHT
  defb b_islandtopleft,    b_lakebottom, b_islandtopright 
  defb b_lakeright,        b_nothing,    b_lakeleft   
  defb b_islandbottomleft, b_laketop,    b_islandbottomright
  defb 255 ; END OF LIST

mapdef_forest:
  defb 3,3 ; WIDTH / HEIGHT
  defb 0, b_buildforest, 0
  defb b_buildforest,b_buildforest,b_buildforest
  defb 0, b_buildforest, 0
  defb 255 ; END OF LIST

mapdef_mount:
  defb 3,3 ; WIDTH / HEIGHT
  defb b_buildmountainrangetop,    b_buildmountainrange,     0
  defb b_buildmountainrangebottom, b_buildmountainrangeleft, b_buildmountainrangeright
  defb 0,                          b_buildmountainrange,     0
  defb 255 ; END OF LIST

mapdef_lake:
  defb 3,3 ; WIDTH / HEIGHT
  defb b_laketopleft,    b_laketop,    b_laketopright
  defb b_lakeleft,       b_lakemid,    b_lakeright
  defb b_lakebottomleft, b_lakebottom, b_lakebottomright
  defb 255 ; END OF LIST

mapdef_smalllake:
  defb 2,2 ; WIDTH / HEIGHT
  defb b_laketopleft,    b_laketopright
  defb b_lakebottomleft, b_lakebottomright
  defb 255 ; END OF LIST

mapdef_roads:
  defb 3,3 ; WIDTH / HEIGHT
  defb 0, b_buildvertroad, 0
  defb b_buildhorizroad,b_buildcrossroad,b_buildhorizroad
  defb 0, b_buildvertroad, 0
  defb 255 ; END OF LIST

; INSERT LAKE INTO LOCAL MAP
; INPUTS
; A  = CHECK OR INSERT
; C  = TILE ID TO START AT
; DE = SPRITE DEFINITION TO INSERT
doinsertmount:
  xor a;ld a,0                   ; SIGNAL THAT WE WANT TO REPLACE EMPTY LAND TILES ONLY
  ld de,mapdef_mount             ; LOAD SPRITE VALUES
  jr insertspritedefinlocalmap
doinsertlake:
  xor a;ld a,0                   ; SIGNAL THAT WE WANT TO REPLACE EMPTY LAND TILES ONLY
  ld de,mapdef_smalllake         ; LOAD SPRITE VALUES
  jr insertspritedefinlocalmap
doinsertforest:
  xor a;ld a,0                   ; SIGNAL THAT WE WANT TO REPLACE EMPTY LAND TILES ONLY
  ld de,mapdef_forest            ; LOAD SPRITE VALUES
  jr insertspritedefinlocalmap
doinsertisland:
  ld a,5                         ; SIGNAL THAT WE WANT TO REPLACE SEA TILES ONLY
  ld de,mapdef_island            ; LOAD SPRITE VALUES
  jr insertspritedefinlocalmap
doinsertroads:
  xor a;ld a,0                   ; SIGNAL THAT WE WANT TO REPLACE LAND TILES ONLY
  ld de,mapdef_roads             ; LOAD SPRITE VALUES
insertspritedefinlocalmap:
  ld (tiletoreplaceminusone-1),a 
  push de
  ld a,1
  ld (checkdigitminusone-1),a    ; SIGNAL FUNCTION THAT WE ARE JUST CHECKIING IF INSERTION IS POSSIBLE
  call insertisland
  pop de
  or a;cp 0                      ; WE CANT INSERT AS THERE IS SOMETHING THERE
  ret z
  xor a;ld a,0
  ld (checkdigitminusone-1),a    ; SIGNAL FUNCTION THAT WE WANT TO INSERT
  call insertisland
ret

insertisland:
  ld hl,gamemap2
  ld l,c
  ld a,(de) ; GET WIDTH OF SPRITE IN B
  ld b,a

  inc de
  inc de    ; SKIP HEIGHT
  insertlakelooprows:
    push bc
    push hl
    insertlakeloop:
      ld a,(de)
      cp 255 ; END OF SPRITE
      jr z,popoffnow
 
      ; CHECK DATA - SELF MODIFY CODE TO CHECK IF INSERTION IS POSSIBLE 
      ; 1 = CHECK IF INSERT POSSIBLE
      ; 0 = DO INSERT
      ld a,0:checkdigitminusone
      or a;cp 0
      jr z,doblockinsert

      ; CHECK IF SEA
      ld a,5:tiletoreplaceminusone
      cp (hl)
      jr nz,popoffnow2
      jr skipblockinsert

      ; INSERT IF SEA
      doblockinsert:
      ld a,(de)
      ld (hl),a

      skipblockinsert:
      inc hl
      inc de
    djnz insertlakeloop
    pop hl
    ld bc,10   ; MOVE TO NEXT ROW IN MAP
    add hl,bc
    pop bc
  jr insertlakelooprows

  popoffnow:
    pop hl
    pop bc
    ld a,1   ; WE CAN INSERT
  ret

  popoffnow2:
    pop hl
    pop bc
    xor a;ld a,0    ; WE CANT INSERT
  ret

; FIND OUT WHAT BLOCK OF TEN SQUARES WE CURRENTLY OCCUPY
; OUTPUTS
; C = X POS IN 10x10 BLOCK 
; B = Y POS IN 10x10 BLOCK
; D = TILE BLOCK NUMBER IN TABLE
getcurrworldmap:
  ; HORIZONTAL POSITION
  ld a,(tilemap_name+1)
  sub "0"
  ld c,a
  ld d,a
  
  ; VERTICAL POSITION
  ld a,(tilemap_name+5)
  sub "0"
  ld b,a
  
  call convxytoblockid
  ; SAVE BLOCK ID
  ld (currblockid),a
ret

; GET VALUE OF CURRENT MAP / ZONE BLOCK
; INPUTS
; HL = MAP OR ZONE TABLE
; OUTPUTS
; A = VALUE OF CURRENT BLOCK
getmapblockid:
  ld b,0
  ld a,(currblockid)
  ld c,a
  add hl,bc
  ld a,(hl)
ret

getcurrworldmapvalue:
  ; GET VALUE OF CURRENT MAP BLOCK TO E
  ld hl,gamemap
  call getmapblockid
  ld (currscreendata),a

  ; GET VALUE OF CURRENT ZONE BLOCK TO E
  ld hl,zonemap
  call getmapblockid
  ld (currzonedata),a
ret
; C = X POS  
; B = Y POS
; D = TILE BLOCK NUMBER IN SECOND MAP TABLE
getcurrlocalmap:
  ld a,(tilemap_name+2)
  sub "0"
  ld c,a
  ld d,a
  ld (localscreenx),a
  ld a,(tilemap_name+6)
  sub "0"
  ld b,a
  ld (localscreeny),a

  call convxytoblockid
  ; SAVE SCREEN ID
  ld (currscreenid),a
ret
getcurrlocalmapvalue:
  ; GET VALUE OF CURRENT TILE TO E
  ld hl,gamemap2
  ld b,0
  ld a,(currscreenid)
  ld c,a
  add hl,bc
  ld a,(hl)
  ld (currscreendata2),a
ret
; CONVERT A BLOCK X, Y COORDINATE TO A BLOCK ID NUMBER
; C = X POS  
; B = Y POS
; OUTPUT
; A = BLOCK ID
blockiddata:
  defb 0,10,20,30,40,50,60,70,80,90

convxytoblockid:
  ld hl,blockiddata
  push bc
  ; GET ROW Y POS
  ld c,b
  ld b,0
  add hl,bc
  ld a,(hl)
  pop bc
  ; ADD COLUMN X POS
  add c
  ret

; INDEXED ON WORLD MAP BLOCK ID
;  defw seamapsea                ; 0
;  defw seamapleftland           ; 1
;  defw seamaptopland            ; 2
;  defw seamaprightland          ; 3
;  defw seamapbottomland         ; 4
;  defw seamaptopleftland        ; 5
;  defw seamaptopleftsea         ; 6
;  defw seamaptoprightland       ; 7
;  defw seamaptoprightsea        ; 8
;  defw seamapbottomleftland     ; 9
;  defw seamapbottomleftsea      ; 10
;  defw seamapbottomrightland    ; 11
;  defw seamapbottomrightsea     ; 12
;  defw seamapland               ; 13

; PIXEL COLOURS OF ZONES FOR LOCAL AND WORLD MAPS

colzones:
        DB      #F0 ; 0  seamapsea
        DB      #FA ; 1  seamapleftland
        DB      #FF ; 2  seamaptopland
        DB      #F5 ; 3  seamaprightland
        DB      #FF ; 4  seamapbottomland
        DB      #FA ; 5  seamaptopleftland
        DB      #F5 ; 6  seamaptopleftsea
        DB      #FF ; 7  seamaptoprightland
        DB      #FA ; 8  seamaptoprightsea
        DB      #FF ; 9  seamapbottomleftland
        DB      #F5 ; 10 seamapbottomleftsea
        DB      #F5 ; 11 seamapbottomrightland
        DB      #FA ; 12 seamapbottomrightsea
        DB      #FF ; 13 seamapland
        DB      #FF ; 14 seamaptopleftriver      
        DB      #FF ; 15 seamapbottomleftriver 
        DB      #FF ; 16 seamapdividinglandtop 
        DB      #FF ; 17 seamapdividinglandbottom 

; F0 BLUE
; FA GREEN BLUE
; F5 BLUE GREEN

cols:
        DB      #FF ;  0 b_nothing 
        DB      #FA ;  1 b_laketopleft
        DB      #F0 ;  2 b_laketop
        DB      #F5 ;  3 b_laketopright
        DB      #FA ;  4 b_lakeleft
        DB      #F0 ;  5 b_lakemid
        DB      #F5 ;  6 b_lakeright
        DB      #FA ;  7 b_lakebottomleft
        DB      #F0 ;  8 b_lakebottom
        DB      #F5 ;  9 b_lakebottomright
        DB      #FF ; 10 b_rivertopleft
        DB      #FF ; 11 b_riververtleft
        DB      #FF ; 12 b_rivertopright
        DB      #FF ; 13 b_riverhoriz
        DB      #FF ; 14 b_riverhoriz2
        DB      #FF ; 15 b_riverbottomleft
        DB      #FF ; 16 b_riverbottomright
        DB      #F0 ; 17 b_islandtopleft
        DB      #F0 ; 18 b_islandtopright
        DB      #F0 ; 19 b_islandbottomleft
        DB      #F0 ; 20 b_islandbottomright
        DB      #C0 ; 21 WHITE HOME
        DB      #3C ; 22 YELLOW TOWN
        DB      #3C ; 23 YELLOW TOWN
        DB      #3C ; 24 YELLOW TOWN
        DB      #3C ; 25 YELLOW TOWN
        DB      #3C ; 26 YELLOW TOWN
        DB      #30 ; 27 RED GAME ARCHERY
        DB      #F3 ; 28 TEAL MAZE
        DB      #F3 ; 29 TEAL MAZE
        DB      #F3 ; 30 TEAL MAZE
        DB      #F3 ; 31 TEAL MAZE
        DB      #30 ; 32 RED - MIDDLE MAZE
        DB      #F3 ; 33 TEAL MAZE
        DB      #F3 ; 34 TEAL MAZE
        DB      #F3 ; 35 TEAL MAZE
        DB      #F3 ; 36 TEAL MAZE
        DB      #30 ; 37 CASTLE - RED
        DB      #CC ; 38 MOUNTAINS - BROWN
        DB      #CC ; 39 MOUNTAINS - BROWN

;b_buildmountainrangetopleft equ 38
;b_buildmountainrangeleft    equ 39
;b_riverseatop               equ 42
;b_riverseabottom            equ 43
;b_buildforest               equ 44
;b_narniahouse               equ 45
;b_mountainblockbottom       equ 46 
;b_landfence                 equ 47
;b_sealandfenceleft          equ 48
;b_sealandfenceright         equ 49

        DB      #30 ; 40 RED
        DB      #30 ; 41 MINE 2 RED

        DB      #FF ; 42 b_riverseatop
        DB      #FF ; 43 b_riverseabottom
        DB      #03 ; 44 b_buildforest
        DB      #30 ; 45 b_narniahouse
        DB      #CC ; 46 b_mountainblockbottom
        DB      #FF ; 47 b_landfence
        DB      #F5 ; 48 b_sealandfenceleft
        DB      #FA ; 49 b_sealandfenceright 
        DB      #CC ; 50 b_buildmountainrange      
        DB      #CC ; 51 b_buildmountainrangeleft    
        DB      #CC ; 52 b_buildmountainrangeright 
        DB      #CC ; 53 b_buildcrossroad      
        DB      #CC ; 54 b_buildvertroad             
        DB      #CC ; 55 b_buildhorizroad         
        DB      #3C ; 56 3C YELLOW
        DB      #FC ; 57 ORANGE
        DB      #03 ; 58 PURPLE
        DB      #C3 ; 59 CYAN
        DB      #0F ; 60 MAUVE
        DB      #CF ; 61 L GREEN
        DB      #33 ; 62 DULL GREEN
        DB      #F3 ; 63 TEAL
        DB      #3F ; 64 GREY
        DB      #FF ; 65 GREEN

; DRAW CURRENT GAME MAPS TO SCREEN
drawmap:
  ld a,(inbuilding) ; DON'T DRAW MAP IF WE ARE IN BUILDING
  or a;cp 0
  ret nz

  ld a,(hasworldmap)
  or a
  jr z,skipworldmap
  ld hl,&c000
  ld de,gamemap
  call drawmap1
  call plotuserposworld

  skipworldmap:
  ld a,(haslocalmap)
  or a
  ret z
  ld hl,&c000+80-10
  ld de,gamemap2
  call drawmap2
  call plotuserposlocal
ret

; DRAW MAP ON SCREEN
; INPUT
; HL = SCREEN POS
; DE = GAME MAP
drawmap1:
  ld b,10
  dorow:
    push de
    call plotmaplineworld
    pop de
    call plotmaplineworld 
    ; DOUBLE LINES FOR READABILITY
  djnz dorow
ret

drawmap2:
  ld b,10
  dorow2:
    push de
    call plotmaplinelocal
    pop de
    call plotmaplinelocal
    ; DOUBLE LINES FOR READABILITY
  djnz dorow2
ret

plotmaplinelocal:
    push hl
    push bc
    ld b,10
    docol:
      call plotitlocalmap
    djnz docol
    donextplotline:
    pop bc
    pop hl  
    push bc
    call scr_next_line_hl
    pop bc
ret

plotmaplineworld:
    push hl
    push bc
    ld b,10
    docol2:
      call plotitworldmap
    djnz docol2
    jr donextplotline

plotitworldmap:
  push hl
  ld hl,colzones
  jr doplotmap
plotitlocalmap:
  push hl
  ld hl,cols
  doplotmap:
  ld a,(de)
  push bc
  ld b,0
  ld c,a
  add hl,bc
  ld a,(hl)
  pop bc
  pop hl
  ld (hl),a
  inc hl
  inc de
ret

worldmapylookup:
  defw &C000
  defw &D000
  defw &E000
  defw &F000
  defw &C050
  defw &D050
  defw &E050
  defw &F050
  defw &C0A0
  defw &D0A0
mapylookup:
  defw &C046
  defw &D046
  defw &E046
  defw &F046
  defw &C096
  defw &D096
  defw &E096
  defw &F096
  defw &C0E6
  defw &D0E6

plotuserposlocal:
  ld hl,mapylookup
  ; ADD ROWS DOWN
  ld a,(localscreeny)
  call vectorlookuphl

  ; ADD BYTES ACROSS
  ld a,(localscreenx)
  plotonmap:
  ld b,0
  ld c,a
  add hl,bc 

  ld (hl),#30
  call scr_next_line_hl
  ld (hl),#30
ret
plotuserposworld:
  ld hl,worldmapylookup
  ; ADD ROWS DOWN
  ld a,(tilemap_name+5)
  sub "0"
  call vectorlookuphl
  ; ADD BYTES ACROSS
  ld a,(tilemap_name+1)
  sub "0"
  jr plotonmap