; OBJECTS THAT CAN BE PLACED DIRECTLY ON MAP

; LIST OF OBJECT IDS
; THESE CORRESPOND TO ORDER OF STRINGS IN INVENTORY DICTIONARY BELOW
inventoryiddictlookup:
  defb rabbitmeat   ; 1 - rabbit meat
  defb peacepipe    ; 2 - peace pipe
  defb moneybag     ; 3 - money bag
  defb feather      ; 4 - feather
  defb letter       ; 5 - love letter
  defb lostson      ; 6 - lost son
  defb flipper_id   ; 7 - flippers
  defb rope_id      ; 8 - rope
  defb parcel       ; 9 - parcel
  defb key          ; 10 - key
  defb map_id       ; 11 - map
  defb 255          ; end of list

; POINTERS TO NAMES OF OBJECTS THAT CAN BE PICKED UP
inventorydict:
  defw str_empty     ; WE NEED THIS SPACE TO MAKE ABOVE LOOKUP WORK
  defw str_rabbitmeat
  defw str_peacepipe
  defw str_moneybag
  defw str_feather
  defw str_loveletter
  defw str_lostson
  defw str_flippers
  defw str_rope
  defw str_parcel
  defw str_key
  defw str_map

str_rabbitmeat:  defb "TASTY MORSEL",255
str_peacepipe:   defb "PEACE PIPE",255
str_feather:     defb "FEATHER",255

str_moneybag:    defb "BAG OF GOLD",255
str_loveletter:  defb "LOVE LETTER",255

str_lostson:     defb "SON",255
str_flippers:    defb "PAIR OF FLIPPERS",255
str_rope:        defb "ROPE",255
str_parcel:      defb "PARCEL",255
str_key:         defb "KEY",255
str_map:         defb "MAP",255

; CHECK PLAYER INVENTORY
; CURRENT LIMITS - ONLY ONE QTY PER ITEM
; MAX INVENTORY - 10 ITEMS
; INPUTS
; B = ITEM ID TO CHECK
; OUTPUTS
; A = 0 = NOT FOUND, 1 = FOUND
checkplayerinv:
  ld a,b        ; BACKUP OBJECT ID IN B FOR CHECKING
  or a;cp 0     ; RETURN IF ITEM IS 0
  jp z,returnfalse
  ld hl,player_inventory
  invloop3:
    ld a,(hl)
    cp 255             ; END OF INVENTORY LIST
    jp z,returnfalse
    cp b  
    jp z,returntrue             ; FOUND ENTRY
    inc hl
    inc hl ;call nextinventoryitem
  jr invloop3

checkplayerinvrand:
  ld hl,player_inventory_rand
  jr invloop3

; B = ITEM ID TO ADD
; QTY ASSUMED 1
addplayerinvrand:
  ; CHECK IF WE HAVE IT ALREADY, IF SO UPDATE QTY
  call checkplayerinvrand
  or a
  jr z,addplayerinvrandnew

  addplayerinvrandupd:
  ld hl,player_inventory_rand
  invloop2:
    ld a,(hl)
    cp 255           ; END OF INVENTORY LIST
    jp z,returnfalse ; WE DIDN'T HAVE ENOUGH ROOM TO PICK UP OBJECT 
    cp b
    jr nz,skiptonextinvitem
    ; INCREMENT ITEM QTY
    inc hl
    inc (hl) 
    jp returntrue      
             
    skiptonextinvitem:
    inc hl
    inc hl
  jr invloop2

addplayerinvrandnew:
  ; UPDATE EXISTING QTY
  ld hl,player_inventory_rand
  invloop6:
    ld a,(hl)
    cp 255           ; END OF INVENTORY LIST
    jp z,returnfalse ; WE DIDN'T HAVE ENOUGH ROOM TO PICK UP OBJECT 
    or a;cp 0        ; EMPTY ENTRY
    jr nz,skiptonextinvitem2
    
    ; ADD NEW ITEM AND QTY
    ld (hl),b
    inc hl
    inc (hl) 
    ; IF WE ADDED BOW OR ARROW, WE NEED TO CHECK POWERS
    jp finishedchangeinventory 

    skiptonextinvitem2:
    inc hl
    inc hl;call nextinventoryitem
  jr invloop6

str_inventoryobject:
   defb 14,1:showinvnumber
   defb ") "
   defb 14,1:showinvqty
   defb " ",11,255

; CHECK IF PLAYER HAS ANYTHING TO SELL
checkinventoryempty:
  ld hl,player_inventory_rand
  checkinventoryemptyloop:
    ld a,(hl)
    cp 255
    jp z,returntrue
    or a              ; WE FOUND SOMETHING
    jp nz,returnfalse
    inc hl
    inc hl          
  jr checkinventoryemptyloop 

; SHOW PLAYERS RANDOM INVENTORY ITEMS
showinventory:
  ld hl,player_inventory_rand
  ld c,255 ; LINE COUNT
  showinventoryloop:
    ld a,(hl)
    cp 255                        ; END OF LIST
    jr z,finishprinttext
    or a
    jr z,skipinvitem

    inc c
    ld b,a                        ; STORE OBJECT ID

    push hl
    inc hl
    ld a,(hl)                     ; GET QTY
    ld (showinvqty-1),a
    call getrandobjectstr         ; GET NAME IN DE

    ld a,c
    inc a
    ld (showinvnumber-1),a          ; SHOW LINE NUMBER

    ld hl,str_inventoryobject
    xor a                         ; DON'T SCROLL UP
    call statusbarprintparagraph
    pop hl

    skipinvitem:
    inc hl
    inc hl
  jr showinventoryloop

finishprinttext:
  ; SCROLL TEXT SO INVENTORY STARTS AT TOP
  ld a,c
  ld (linecount),a
  call scrolltopofpage
  jp checknokeyspressed

; UPDATE PLAYER INVENTORY - ADD
; CURRENT LIMITS - ONLY ONE QTY PER ITEM
; MAX INVENTORY - 10 ITEMS
; INPUTS
; B = ITEM ID TO ADD
; C = TILENUMBER
addplayerinv:
  ld a,b
  cp moneybag ; IF PICKED UP MONEY BAG
  jr z,rewardgold2
  cp arrow    ; IF PICKED UP ARROWS
  jr z,rewardarrows

  ld hl,player_inventory
  invloop:
    ld a,(hl)
    cp 255           ; END OF INVENTORY LIST
    jp z,returnfalse ; WE DIDN'T HAVE ENOUGH ROOM TO PICK UP OBJECT 
    or a;cp 0        ; EMPTY ENTRY
    jr z,addinvitem2
    inc hl 
    inc hl
    ;call nextinventoryitem
  jr invloop

  addinvitem2:
    ld (hl),b
    inc hl
    ld (hl),1
    call updatestatusbaricon ; DISPLAY ITEM WE HAVE BEEN GIVEN

    finishedchangeinventory: ; CHECK POWERS
    call checkpowers
  jp returntrue

rewardgold2:
  push bc              ; STORE B AS THIS CONTAINS OUR SPRITE ID
  ld hl,(player_cash2)
  ld bc,10
  add hl,bc
  ld (player_cash2),hl
  pop bc
  dostatusbar:
  push bc
  call updatestatusbarmoney
  pop bc
ret
rewardarrows:
  ld b,1 ; LOAD ARROW ID
  jp addplayerinvrand
  ;ld a,(player_arrows)
  ;add 10
  ;ld (player_arrows),a
  ;jr dostatusbar

; UPDATE PLAYER INVENTORY - REMOVE
; MAX INVENTORY - 5 ITEMS
; ITEM ID, QTY
; INPUTS
; B = ITEM ID TO REMOVE
remplayerinvrand:
  ld hl,player_inventory_rand
  jr invloop4
remplayerinv:
  ld hl,player_inventory
  invloop4:
    ld a,(hl)
    cp 255 ; END OF INVENTORY LIST
    jr z,finishedchangeinventory
    cp b   ; FOUND ENTRY
    jr z,decrementplayeritem;remplayerinvitem
    inc hl
    inc hl;call nextinventoryitem
  jr invloop4

  decrementplayeritem:
    inc hl
    dec (hl)
    ; CHECK IF WE HAVE NONE LEFT, IF SO REMOVE ENTRY
    xor a
    cp (hl)
    ret nz
    dec hl
    ld (hl),0
  ret

;nextinventoryitem:
;  inc hl
;  inc hl
;ret