;; width of a tile
tile_width equ 8
tile_width_bytes equ tile_width/2
font_width_bytes equ 4/2
;; height of a tile
tile_height equ 16
font_height equ 5
;; number of bytes used by tile (each tile uses same 
;; number of bytes and stored uncompressed in a formmn
;; that can be poked direct to screen)
tile_data_size equ tile_width_bytes*tile_height
font_data_size equ font_width_bytes*font_height

;ScreenSize equ &4000
;; width of screen in pixels
scr_width_pixels         equ tile_width*20
;scr_right_side equ scr_width_pixels-4
;; height of screen in pixels
scr_height_pixels        equ tile_height*10
scr_width_bytes          equ scr_width_pixels/2
scr_width_bytes_right    equ scr_width_bytes-4
scr_height_pixels_bottom equ scr_height_pixels-16
;; width of screen in tiles
tilemap_width_tiles      equ scr_width_bytes/tile_width_bytes
;; height of screen in tiles
tilemap_height_tiles     equ scr_height_pixels/tile_height
tilemap_total_tiles      equ tilemap_width_tiles*tilemap_height_tiles
tilemap_win_tiles        equ tilemap_width_tiles*tilemap_height_tiles

;; firmware functions we use

scr_set_mode    equ &bc0e
scr_set_border  equ &bc38
scr_set_ink     equ &bc32
mc_wait_flyback equ &bd19

;km_test_key     equ &bb1e ; CHECK FOR KEY AND WAIT FOR RESPONSE
;km_read_key     equ &bb1b ; CHECK FOR KEY BUT DON'T WAIT FOR RESPONSE
;km_read_char    equ &bb09
;km_reset        equ &bb03
;km_wait_char    equ &bb06
;km_set_delay    equ &bb3f
;km_wait_key     equ &bb18

;txt_set_paper     equ &bb96
;gra_set_paper     equ &bbe4
;gra_set_pen       equ &bbde
;gra_wr_char       equ &bbfc
;gra_move_absolute equ &bbc0

;kl_time_please    equ &bd0d
;kl_time_set       equ &bd10


org &1200
nolist
jp start

detectram:
;  ld hl,&c000
;  ld a,255
;  ld (hl),a

;  ld a,&C3
;  call bankswitch
;  ld a,(hl)
;  cp 255
;  ret z

;  ld a,&C0
;  call bankswitch
  ; LOADER PUTS A 255 HERE IF WE HAVE 128k
  ;ld a,(&1FFF)
  ;cp 255
  ;ret nz ; 64k

  ; 128k
  ;ld hl,onetwoeightk
  ;call statusbarprintparagraph

  ;ld b,10
  ;jp updatestatusbaricon
  

; INPUT
; A = BANK TO SWITCH
; &C4,&C5,&C6,&C7
; SWAPS &4000-&7FFF WITH EXTRA BANKS 1-4
;bankswitch:
;  ld b,&7F
;  out (c),a
;ret

;bankswitchsprites:
;  ld a,&C4
;  jr bankswitch

;bankswitchcode:
;  ld a,&C0
;  jr bankswitch

; READ CODE MODULES
read "definitions.asm"      ; DEFINITIONS FOR 64k OR 128k
read "examplekeyboard.asm"  ; JOYSTICK READING CODE
read "pseudorandom.asm"     ; PSUEDO RANDOM GENERATOR
read "graphics.asm"         ; MISC GRAPHICS FUNCTIONS
read "statusbar.asm"        ; STATUS BAR RENDERING
read "daynight.asm"         ; CONTROL DAY AND NIGHT PALETTE
read "speedup.asm"          ; MISC ASM SPEED UP
read "populatetilemap2.asm" ; PROCEDURAL MAP GENERATION
read "zonedefs.asm"         ; ZONE DEFINITION TABLES
read "dictionary.asm"       ; DICTIONARY OF WORDS
read "inventory.asm"        ; INVENTORY CODE
read "shop.asm"             ; SHOP CODE
read "populaterooms.asm"    ; PROCEDURAL ROOM GENERATION
read "spritemask.asm"       ; SPRITE MASK ROUTINE
read "spritemovement.asm"   ; SPRITE MOVEMENT ROUTINES
read "gengamemap.asm"       ; GENERATE GAME MAP
ifdef sixtyfourk
  ;org &4000
  read "tilegraphicsmain.asm"     ; SPRITE GRAPHICS
  read "tilegraphicssprite.asm"     ; SPRITE GRAPHICS
  read "tilegraphics.asm"     ; SPRITE DATA
  read "seamaps.asm"          ; SPRITE DATA
else
  ; SWITCHABLE BLOCK OF MEMORY BETWEEN 4000 AND 7FFF
  ;org &4000
  ;tile_gfx equ &4000
  ;defb "SPRITEDEFS"
  read "tilegraphicsmain.asm"     ; SPRITE GRAPHICS
  read "tilegraphicssprite.asm"     ; SPRITE GRAPHICS
  ;org &8000
  ;defb "SPRITEDATA"
  read "tilegraphics.asm"     ; SPRITE DATA
  read "seamaps.asm"          ; SPRITE DATA
endif

;org &8000
read "generatetiles.asm"    ; SPRITE GRAPHICS
read "sound3.asm"           ; SOUND EFFECTS
read "quests2.asm"           ; QUEST DATA
read "randomquests2.asm"     ; RANDOM QUEST DATA
read "storylines.asm"       ; STORY LINE DATA
read "mazedata.asm"         ; STATIC ROOM GENERATION
read "data.asm"             ; SPRITE DATA
read "collision.asm"        ; SPRITE MOVEMENT ROUTINES

;musicdata:
;  defb 43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65

;pitchoffset:
;  defb 0

start:
  ld c,&ff            ; DISABLE ALL ROMS
  ; INITIATE DATA AREAS OF MEMORY, STRINGS, ETC
  call initiatevars
  ; SET TIME FOR PSEUDO RANDOM GENERATION
  ;ld de,0
  ;ld hl,0
  ;call kl_time_set

  xor a; ld a,0
  call scr_set_mode
  ;call setmode
  ;; set border to black
  ld bc,0
  call scr_set_border

  ;ld h,1
  ;ld l,1
  ;call km_set_delay

  ;; set pens
  ld hl,pens
  ld b,16
  xor a      ;; initial pen index
  set_pens:
    push bc
    push af
    ld c,(hl)   ;; B,C = inks for pen. If they are the same then no flashing
    inc hl
    ld b,c;(hl)
    ;inc hl
    push hl
    call scr_set_ink
    pop hl
    pop af
    pop bc
    inc a      ;; increment pen index
  djnz set_pens

  ; GENERATE LOOKUP TABLES
  call init_mask_table
  call make_scr_table
  ;ld ix,tile_ptr_table
  ;ld hl,tile_gfx        ;; start of tile pixels (tile 0)
  ;ld de,tile_data_size  ;; size of each tile (all tiles same size in bytes)
  ;ld b,num_tiles        ;; number of tiles to generate information for
  ;call make_ptr_table
  ld ix,letters_ptr_table
  ld hl,letters_gfx     ;; start of tile pixels (tile 0)
  ld de,font_data_size  ;; size of each tile (all tiles same size in bytes)
  ld b,59               ;; number of tiles to generate information for
  call make_ptr_table

  ; DRAW NON CHANGE GRAPHICS
  call ClearStatusBar
  call drawstatusbar

  ld hl,intro
  ld a,0 ; DON'T SCROLL UP
  call statusbarprintparagraph
  ;call waitspace
  ;call detectram
  ; GENERATE TILEMAP
  call marktilemapchanged
  ;ld a,1
  ;ld (tilemap_changed),a
  ;???????????????
  ;ld hl,screentext1
  ;call statusbarprintparagraph

  ;xor a
  ;ld (tilemap_cur),a
  ;ld a,1
  ;ld (tilemap_changed),a

  ;call km_wait_key
  ;call mc_wait_flyback
  
  ; GET INTERUPT HANDLER TO CHECK SPRITE UPDATES
  ;di
  ;ld a,&C3
  ;ld (&0038),a
  ;ld hl,interupthandler   
  ;ld (&0039),hl
  ;ei
  ;ld hl,newtext
  ;call statusbarprintparagraph
 
;; the main loop which draws and updates sprite

main_loop:
  ;; check keys and update sprite position
  call check_keys
  call interupthandler
  call check_tilemap

  ; UPDATE ARROWS IN MAIN ROUTINE AS IT NEEDS TO MOVE FAST
  ld ix,mcparrowdata
  call updatecharactersprites

  ld a,(interuptcounter_plus1-1)
  or a;cp 0
  jr nz,main_loop
  ld a,10
  ld (interuptcounter_plus1-1),a

  ; UPDATE SPRITE CHARACTERS IN SUB ROUTINE AS THEY NEED TO MOVE SLOW
  ; LOAD CHARACTER TABLE DATA
  ld ix,mcps
  call updatecharactersprites
  call updatetileanims
  call updatestatusbartime
  ;call checktimeofday

  ; GO TO NEXT MUSICAL NOTE
  ;ld a,(pitchoffset)
  ;inc a
  ;cp 16
  ;jr z,resetpitchoffset

  ;dosound:
  ;ld (pitchoffset),a
  ;ld hl,musicdata
  ;ld b,0
  ;ld c,a
  ;add hl,bc
  ;ld a,(hl)
  ;call mainsound
  ;ld a,0
  ;jr main_loop

;resetpitchoffset:
;  ld a,0
;  jr dosound

jr main_loop
;ei

interupthandler:
  ;ex af, af'
  ld a,15 :interuptcounter_plus1
  dec a
  ld (interuptcounter_plus1-1),a
  ;ex af, af'
  ;ei         ; ENABLE INTERUPTS AGAIN
ret

; REDRAW SCREEN ON TILEMAP CHANGE
check_tilemap:
  ld a,(tilemap_changed)
  or a;cp 0
  ret z
  call deactivatemcpsdoublebuffer
  call deactivateanimations
  ;call clearunusedquests             ; CLEAR UNUSED QUEST IDS SO WE CAN CREATE MORE
  call gentilemapa
  call spawnrandommcps
  call markallcharacterspritesredraw
  xor a;ld a,0
  ld (tilemap_changed),a
ret
  
;new_quest:
;  ld hl,quest_wagon
;  call statusbarprintparagraph
;ret

;; check if a key has been pressed and perform action if it has
;check_keys2:
  ;ld a,0	        ;; cursor up
  ;call km_test_key 
  ;ret nc
  ;cp "f"
  ;jr z,tilemap_up
  ;cp "c"
  ;jr z, tilemap_down
  ;cp "z"
  ;jr z, tilemap_left
  ;cp "x"
  ;jr z, tilemap_right
;  ld a,0              ;; cursor up
;  call km_test_key
;  jr nz, player_up
;  ld a,2              ;; cursor down
;  call km_test_key
;  jr nz, player_down
;  ld a,8              ;; cursor left
;  call km_test_key
;  jr nz, player_left
;  ld a,1              ;; cursor right
;  call km_test_key
;  jp nz, player_right
  ;cp "q"
  ;jr z, new_quest
;ret



marktilemapchanged:
  ld a,1
  ld (tilemap_changed),a
ret

togglemap:
  ; CHECK IF WE HAVE MAP
  ld a,(viewmap)
  xor 255
  ld (viewmap),a

  ld b,3
  call checkplayerinvrand
  ld (hasworldmap),a                 ; TOGGLE MAP VIEW
  ld b,4
  call checkplayerinvrand
  ld (haslocalmap),a                 ; TOGGLE MAP VIEW
  call checknokeyspressed            ; WAIT SO WE DON'T GO REALLY FAST

  ld a,(viewmap)
  or a
  ret z
  call drawmap
ret

;; based on current tilemap, move to next tilemap up
set_tilemap_view:
  ld a,(ix+4)
  ld (tilemap_cur),a
  ld a,(ix+5)
  ld (tilemap_cury),a
  jr marktilemapchanged


tilemap_draw:
  ; CHECK IF DRAWING JUST SEA, IF SO SKIP
  ld a,(currscreendata2)
  cp 5 ; ALL SEA, SKIP
  ret z
  ld hl,0  ;; H = X screen coord of top-left of tilemap
           ;; L = Y screen coord of top-left of tilemap
           ;; width and height of tilemap in tiles
  ld b,tilemap_height_tiles
tilemap_draw_y:
  push hl
  push bc
  ;; calc screen address
  ;; from x,y coord
  call get_scr_addr

  ; UNWRAPPED LOOP
  ld bc,tile_width_bytes ;; increment by width of tile in bytes
  call tilemap_draw_x
  call tilemap_draw_x
  call tilemap_draw_x
  call tilemap_draw_x
  call tilemap_draw_x
  call tilemap_draw_x
  call tilemap_draw_x
  call tilemap_draw_x
  call tilemap_draw_x
  call tilemap_draw_x
  call tilemap_draw_x
  call tilemap_draw_x
  call tilemap_draw_x
  call tilemap_draw_x
  call tilemap_draw_x
  call tilemap_draw_x
  call tilemap_draw_x
  call tilemap_draw_x
  call tilemap_draw_x
  call tilemap_draw_x
  
  pop bc
  pop hl
  ;; H = x coordinate
  ;; L = y coordinate

  ;; update Y coordinate for next tilemap line
  ld a,l
  add tile_height
  ld l,a
  djnz tilemap_draw_y
ret

drawwater:
  ld a,&0F ; BLUE
  push de
  push hl
  call ClearTile 
  pop hl
  pop de
  jr skip

tilemap_draw_x:
  ;; read tile index
  ld a,(de)
  or a;cp 0
  jr z,skip ;; SKIP DRAWING IF TILE NUM IS 0
  cp 100   ; WATER - DRAW WATER TILE MANUALLY - THIS SAVES 64 BYTES
  jr z,drawwater

  ; CHECK OBJECT TILE MAP TO SEE IF WE NEED TO FLIP SPRITE VERTICALLY
  ; 1 = NORMAL OBJECT
  ; 4 = VERTICALLY FLIPPED OBJECT
  ; 5 = VERTICALLY FLIPPED NON OBSTRUCTIONAL OBJECT
  ld a,e
  push de
  ld de,tilemap2
  ld e,a
  ld a,(de)
  pop de
  cp 4
  jr z,spriteflippedvert
  cp 246 ; FLIPPED NON OBSTRUCTING SPRITE
  jr z,spriteflippedvert

  ld a,(de) ; LOAD OUR TILE GRAPHIC ID AGAIN

  push hl
  push de
  ld de,tile_ptr_table
  ; FLIP TILE IF IT IS OVER 200
  ; SUBTRACT 200 TO GET TILE GRAPHIC TO FLIP
  sub 201
  jr c,spritenotflipped:

  call get_tile_ptr               ;; lookup ptr to tile pixels
  call tile_restore_flip          ;; now draw tile    
  pop de
  pop hl
  jr skip

  spritenotflipped:
  add 200
  call get_tile_ptr               ;; lookup ptr to tile pixels
  call tile_draw                  ;; now draw tile  
  pop de
  pop hl
  jr skip

  spriteflippedvert:
  ld a,(de)
  push hl
  push de
  ld de,tile_ptr_table
  ; FLIP TILE IF IT IS OVER 200
  ; SUBTRACT 200 TO GET TILE GRAPHIC TO FLIP
  sub 201
  jr c,spritenothorizflipped:

  call get_tile_ptr               ;; lookup ptr to tile pixels
  call tile_restore_flip_vflip    ;; now draw tile    
  pop de
  pop hl
  jr skip

  spritenothorizflipped:
  add 200
  call get_tile_ptr               ;; lookup ptr to tile pixels
  call tile_restore_vflip         ;; now draw tile  
  pop de
  pop hl

  skip:
  inc e
  ;; HL = screen address
  add hl,bc
ret


;; IN:
;; A = tile index
;; DE = tile_ptr_table
;; OUT:
;; DE = tile pixel data
get_tile_ptr:
  dec a ; TILEMAPS START FROM 1 INSTEAD OF 0
get_tile_ptr_text:
  push hl
  ld l,a
  ld h,0
  add hl,hl
  add hl,de
  ld e,(hl)
  inc hl
  ld d,(hl)
  pop hl
ret

;; From our x,y coordinates generate
;; a screen address for top-left of sprite to be drawn at.

;; IN:
;; H = x byte coord
;; L = y line
;; OUT:
;; HL = screen address
get_scr_addr:
  push bc
  push de
  ld c,h    ; STORE X BYTE

  ld h,0    ; GET MEMORY ADDRESS OF Y LINE
  add hl,hl
  ld de,scr_addr_table
  add hl,de
  ld a,(hl)
  inc hl
  ld h,(hl)
  ld l,a

  ld b,0    ; ADD X BYTE TO Y LINE ADDRESS
  add hl,bc
  pop de
  pop bc
ret

;; generate table of screen addresses
;; one address per scanline. The address
;; is for the first byte of each line.
make_scr_table:
  ld ix,scr_addr_table ;; address to store table
  ld hl,&c000          ;; start address of first scanline
  ld b,200             ;; number of scanlines on screen
  mst1:
    ld (ix+0),l
    ld (ix+1),h
    inc ix
    inc ix
    push bc
    call scr_next_line_hl
    pop bc
  djnz mst1
ret

;; generate table/array of tile pixel ptrs
;; so that we can access tile pixels quickly based
;; on tile index 
make_ptr_table:
  mtpt1:
    ld (ix+0),l
    ld (ix+1),h
    inc ix
    inc ix
    add hl,de
  djnz mtpt1
ret

; GRAB FROM SCREEN HL AND STORE IT IN DE BUFFER
tile_grab:
  ld b,tile_height
tile_grab_height:
  push bc
  push hl

  ldi
  ldi
  ldi
  ldi

  pop hl
  call scr_next_line_hl
  pop bc
  djnz tile_grab_height
ret

; GRAB FROM DE BUFFER AND RESTORE TO SCREEN
; INPUTS
; DE = TILE GRAPHIC
; HL = SCREEN LOCATION
tile_restore:
  ld b,tile_height
tile_restore_height:
  push bc
  push hl

  ex de,hl
  ldi
  ldi
  ldi
  ldi
  ex de,hl

  pop hl
  call scr_next_line_hl
  pop bc
  djnz tile_restore_height
ret

; GRAB FROM DE BUFFER AND RESTORE TO SCREEN UPSIDE DOWN
; INPUTS
; DE = TILE GRAPHIC
; HL = SCREEN LOCATION
tile_restore_vflip:
  ex de,hl
  push bc
  ld bc,60 ; MOVE TO LAST ROW OF PIXELS IN SPRITE DATA
  add hl,bc
  pop bc
  ex de,hl
  ld b,tile_height
tile_restore_height_vflip:
  push bc
  ld bc,mask_table
  push hl
  push de ; STORE POSITION IN SPRITE DATA

  ld a,(de)
  call maketransparent;swappixels
  ld (hl),a
  inc hl
  inc e
  ld a,(de)
  call maketransparent;swappixels
  ld (hl),a
  inc hl
  inc e
  ld a,(de)
  call maketransparent;swappixels
  ld (hl),a
  inc hl
  inc e
  ld a,(de)
  call maketransparent;swappixels
  ld (hl),a
  inc hl

  ;ex de,hl
  ;ldi
  ;ldi
  ;ldi
  ;ldi
  ;ex de,hl
  pop de  ; RESTORE POSITION IN SPRITE DATA AND MOVE BACK ONE ROW
  dec e
  dec e
  dec e
  dec e

  pop hl
  call scr_next_line_hl
  pop bc
  djnz tile_restore_height_vflip
ret

tile_restore_flip:
  ld b,tile_height
  ; MOVE TO RIGHT SIDE OF SPRITE
  inc hl
  inc hl
  inc hl
tile_restore_height_flip:
  push bc
  ld bc,mask_table
  push hl

  call maketransparentswapped
  ld (hl),a
  dec hl
  inc e
  call maketransparentswapped
  ld (hl),a
  dec hl
  inc e
  call maketransparentswapped
  ld (hl),a
  dec hl
  inc e
  call maketransparentswapped
  ld (hl),a
  inc e

  pop hl
  call scr_next_line_hl
  pop bc
  djnz tile_restore_height_flip
ret

; INPUT
; DE = PIXEL BYTE TO SWAP
; OUTPUT
; A = PIXEL DATA TO WRITE
;swappixels:
;  ld a,(de)
;  ; SWAP PIXELS IN BYTE
;  or a;cp 0      ; DO NOT SWAP IF WE JUST HAVE AN EMPTY PIXEL
;  ret z
;  and %01010101  ; REMOVE LEFT PIXEL 
;  sla a          ; SHIFT TO LEFT
;  ld c,a         ; BACKUP TO C
;  ld a,(de)    
;  and %10101010  ; REMOVE RIGHT PIXEL
;  srl a          ; SHIFT TO RIGHT
;  or c           ; INSERT SHIFTED PIXEL 
;  ; WRITE TO SCREEN
;  ld (hl),a
;ret

; INPUTS 
; BC = MASK TABLE
; DE = PIXEL DATA
; HL = SCREEN DATA
maketransparentswapped:
  ld a,(de)
  or a;cp 0 ; DO NOT SWAP IF WE JUST HAVE AN EMPTY PIXEL
  jr z,maketransparent
  ; SWAP PIXELS IN BYTE
  and %01010101    ; REMOVE LEFT PIXEL 
  sla a            ; SHIFT TO LEFT
  ld c,a           ; BACKUP TO C
  ld a,(de)    
  and %10101010    ; REMOVE RIGHT PIXEL
  srl a            ; SHIFT TO RIGHT
  or c             ; INSERT SHIFTED PIXEL 
  maketransparent:
  ld c,a           ;; C = byte of sprite pixel data/look-up table value
                   ;; BC = address (in look-up table) of mask corresponding to this sprite pixel data
  ld a,(bc)        ;; lookup mask from table
  and (hl)         ;; mask pixels on screen (remove pixels which will be replaced)
  or c             ;; combine with sprite pixel data
ret

tile_restore_flip_vflip:
  ld b,tile_height
  ; MOVE TO END OF SPRITE DATA
  ex de,hl
  push bc
  ld bc,63
  add hl,bc
  pop bc
  ex de,hl
tile_restore_height_flip_vflip:
  push bc
  ld bc,mask_table
  push hl

  call maketransparentswapped
  ld (hl),a
  inc hl
  dec e
  call maketransparentswapped
  ld (hl),a
  inc hl
  dec e
  call maketransparentswapped
  ld (hl),a
  inc hl
  dec e
  call maketransparentswapped
  ld (hl),a
  dec e

  pop hl
  call scr_next_line_hl
  pop bc
  djnz tile_restore_height_flip_vflip
ret

; GRAB FROM DE BUFFER AND RESTORE TO SCREEN
tile_restore_scroll:
  ld b,40
  call tile_restore_height
ret

;; TILE DRAW ROUTINE UNWRAPPED. THIS WILL ONLY DRAW
;; SPRITES THAT ARE 8 PIXELS WIDE, BUT IT IS FASTER
;; THAN THE OLD ROUTINE.
;; IN:
;; DE = address of tile pixel data
;; HL = screen address

tile_draw_submerged:
  ld bc,&0050             ; DRAW SUBMERGED
  add hl,bc
  ld b,8
  jr tile_draw_height

tile_draw:
  ld b,tile_height

tile_draw_height:
  push bc
  ld bc,mask_table
  push hl

  ld a,(de)  ;; read from pixels
  ; TRANSPARENCY - COLOUR 0
  call maketransparent
  ld (hl),a        ;; write result to screen 
  inc hl ; WAS INC L, BUT DOESN'T WORK OVER 255?
  inc e

  ld a,(de)  ;; read from pixels
  ; TRANSPARENCY - COLOUR 0
  call maketransparent
  ld (hl),a        ;; write result to screen 
  inc hl
  inc e

  ld a,(de)  ;; read from pixels
  ; TRANSPARENCY - COLOUR 0
  call maketransparent
  ld (hl),a        ;; write result to screen 
  inc hl
  inc e

  ld a,(de)  ;; read from pixels
  ; TRANSPARENCY - COLOUR 0
  call maketransparent
  ld (hl),a        ;; write result to screen 
  inc e

  pop hl
  call scr_next_line_hl
  pop bc
  djnz tile_draw_height
ret

tile_draw_flip_submerged:
  ld bc,&0050             ; DRAW SUBMERGED
  add hl,bc
  ld b,8
  inc hl ; MOVE TO RIGHT SIDE OF SPRITE AND WORK BACKWARDS
  inc hl
  inc hl
  jr tile_draw_height_flip

tile_draw_flip:
  ld b,tile_height
  inc hl ; MOVE TO RIGHT SIDE OF SPRITE AND WORK BACKWARDS
  inc hl
  inc hl

tile_draw_height_flip:
  push bc
  ld bc,mask_table
  push hl

  ; TRANSPARENCY - COLOUR 0
  call maketransparentswapped
  ld (hl),a        ;; write result to screen 
  dec hl ; WAS INC L, BUT DOESN'T WORK OVER 255?
  inc e

  ; TRANSPARENCY - COLOUR 0
  call maketransparentswapped
  ld (hl),a        ;; write result to screen 
  dec hl
  inc e

  ; TRANSPARENCY - COLOUR 0
  call maketransparentswapped
  ld (hl),a        ;; write result to screen 
  dec hl
  inc e

  ; TRANSPARENCY - COLOUR 0
  call maketransparentswapped
  ld (hl),a        ;; write result to screen 
  inc e

  pop hl
  call scr_next_line_hl
  pop bc
  djnz tile_draw_height_flip
ret

;; TILE DRAW ROUTINE UNWRAPPED. THIS WILL ONLY DRAW
;; SPRITES THAT ARE 4 PIXELS WIDE, BUT IT IS FASTER
;; THAN THE OLD ROUTINE.
;; IN:
;; DE = address of tile pixel data
;; HL = screen address
font_draw:
  ld b,5 ; PIXELS HIGH
font_draw_height:
  push bc
  push hl
  ex de,hl
  ldi
  ldi
  ex de,hl
  pop hl
  call scr_next_line_hl
  pop bc
  djnz font_draw_height
ret
list
defb "CHRIS2"