zone_empty                   equ 0
zone_forest                  equ 1
zone_indian                  equ 2
zone_knights                 equ 3
zone_western                 equ 4
zone_tropical                equ 5
zone_sea                     equ 6
zone_winter                  equ 7
zone_autumn                  equ 8
zone_westernindian           equ 9
zone_forestmountain          equ 10
zone_westernmountain         equ 11
zone_westerntropical         equ 12
zone_westerntropicalmountain equ 13

; FOR GENERATING CLIMATES
; WORLD MAP DIVIDED UP INTO 10x10 GRID
; EACH NUMBER IS A ZONE NUMBER FROM ABOVE LIST
zonemap:
  defb 1,  0,  0,  0,  0,  2, 10, 10,  2,  2  ; 000
  defb 0,  6,  6,  4,  4,  2,  2, 10,  2,  2  ; 010
  defb 0,  6,  6,  4,  4,  9,  2,  2,  2,  2  ; 020
  defb 0,  0,  0,  4,  4, 11,  9,  2,  2,  2  ; 030
  defb 7,  7,  7,  4,  9, 11,  9,  2,  2,  2  ; 040
  defb 7,  7,  7,  7, 12, 13,  9,  1,  5,  0  ; 050
  defb 0,  7,  7,  7,  1,  1,  1,  1,  5,  0  ; 060
  defb 8,  8,  1,  1,  1,  1,  1,  1,  5,  0  ; 070
  defb 8,  8,  5,  5,  5,  5,  5,  5,  5,  0  ; 080
  defb 0,  0,  0,  0,  0,  0,  0,  0,  0,  0  ; 090

randomtownbyzone:
  defb b_nothing               ;  0 EMPTY SEA
  defb b_randomtownplains      ;  1 PLAINS
  defb b_randomtownforest      ;  2 PRAIRIE
  defb b_randomtownindian      ;  3 FOREST
  defb b_randomtownwestern     ;  4 WESTERN COWBOY
  defb b_nothing               ;  5 SEA
  defb b_randomtownforest      ;  6 TROPICAL
  defb b_randomtownknights     ;  7 WINTER
  defb b_randomtownplains      ;  8 AUTUMN
  defb b_randomtownindian      ;  9 WESTERN INDIAN
  defb b_randomtownforest      ; 10 FOREST MOUNTAIN
  defb b_randomtownindian      ; 11 MOUNTAIN WESTERN INDIAN
  defb b_randomtownindian      ; 12 WESTERN TROPICAL INDIAN
  defb b_randomtownindian      ; 13 WESTERN TROPICAL MOUNTAIN INDIAN

zone_transp_col:
  ; ZONE ID, DAY COL, NIGHT COL
  defb 9,  0     ; 0 zone_empty,PLAINS - GREEN
  defb 9,  0     ; 1 zone_forest,FOREST - GREEN
  defb 9,  0     ; 2 zone_indian,INDIAN - GREEN
  defb 9,  0     ; 3 zone_knights,KNIGHTS - GREEN
  defb 15, 0     ; 4 zone_western,WESTERN - ORANGE
  defb 9,  0     ; 5 zone_tropical,SEA - MAUVE
  defb 9,  0     ; 6 zone_sea,SEA - BLUE
  defb 9,  0     ; 7 zone_sea,WINTER - BLUE
  defb 15, 0     ; 8 zone_autumn,AUTUMN - YELLOW
  defb 15, 0     ; 9 zone_westernindian,WESTERN - ORANGE
  defb 9,  0     ; 10 zone_forestmountain,FOREST - GREEN
  defb 15, 0     ; 11 zone_westernmountainindian,WESTERN - ORANGE
  defb 15, 0     ; 12 zone_westerntropical,WESTERN - ORANGE
  defb 15, 0     ; 13 zone_westerntropicalmountain,WESTERN - ORANGE

dirtbyzone:
  defw donormalclimate    ;  0 PLAINS
  defw donormalclimate    ;  1 PRAIRIE
  defw donormalclimate    ;  2 FOREST
  defw donormalclimate    ;  3 KNIGHTS
  defw dowesternclimate   ;  4 WESTERN COWBOY
  defw donormalclimate    ;  5 SEA
  defw donormalclimate    ;  6 TROPICAL
  defw dowinterclimate    ;  7 WINTER
  defw donormalclimate    ;  8 AUTUMN
  defw dowesternclimate   ;  9 WESTERN INDIAN
  defw donormalclimate    ; 10 FOREST MOUNTAIN
  defw dowesternclimate   ; 11 WESTERN MOUNT INDIAN
  defw dowesternclimate   ; 12 WESTERN TROPICAL
  defw dowesternclimate   ; 13 WESTERN TROPICAL MOUNTAIN

; CLIMATE EFFECTS
makeallnotsnow:
  ld a,&C0
  ld (checkpixelgreen-1),a
  ld a,&33
  ld (checkpixelwhite-1),a
  call maketreessnow
ret

makeallsnow:
  ld a,&33
  ld (checkpixelgreen-1),a
  ld a,&C0
  ld (checkpixelwhite-1),a
  call maketreessnow
ret

; MODIFY SPRITE, CHANGE COLOUR OF PIXELS FROM GREEN TO WHITE OR VICE VERSA
maketreessnow:
  ld hl,g_mytree1
  ; 5 SPITES TO RECOLOUR
  ld b,6
  changecolloop:
    push bc
    call changecolonesprite
    pop bc
  djnz changecolloop
ret
    
changecolonesprite:
  ld b,64 ; BYTES IN SPRITE/S * NUM OF SPRITES TO RECOLOUR
  snowloop:
    ld a,(hl)
    cp &33:checkpixelgreen
    jr nz,nochangepixelcol
    ld (hl),&C0:checkpixelwhite
    nochangepixelcol:
    inc hl
  djnz snowloop
ret
checkscreenzoneclimate:
  ; CHECK IF WE HAVE MOVED SCREEN ZONES
  ld a,(currblockid)
  ld hl,lastblockid
  cp (hl)
  ret z
  ld (lastblockid),a
  call genlocalgamemap ; PSEUDO RANDOM GENERATE LOCAL MAP SINCE WE HAVE MOVED BLOCKS
  call resetpseudorandomnormal

  ; INPUTS 
  checkclimate:
  ld a,(currzonedata)
  ld hl,dirtbyzone
  jp vectortablelaunchcode

  donormalclimate:
    call createmygrass
    jp makeallnotsnow 
  dowesternclimate:
    call createmydirt
    jp makeallnotsnow
  dowinterclimate:
    call createmygrass
    jp makeallsnow

; INSERT NON ESSENTIAL SPRITES INTO MAP
; INDEXED BY ZONE MAP ID VALUE
landscapetable:
  defw domyplains           ;  0 PLAINS
  defw domyprairie          ;  1 PRAIRIE
  defw domyforest           ;  2 FOREST
  defw domymountain         ;  3 KNIGHTS
  defw domywestern          ;  4 WESTERN COWBOY
  defw domylake             ;  5 SEA
  defw domytropical         ;  6 TROPICAL
  defw domyforest           ;  7 WINTER
  defw domyplains           ;  8 AUTUMN
  defw domywestern          ;  9 WESTERN INDIAN
  defw domyforestmountain   ; 10 FOREST MOUNTAIN
  defw domywesternmountain  ; 11 WESTERN MOUNTAIN INDIAN
  defw domytropical         ; 12 WESTERN TROPICAL INDIAN
  defw domytropicalmountain ; 13 WESTERN TROPICAL MOUNTAIN

; INDEXED BY ZONE MAP ID VALUE
; INSERT NON ESSENTIAL GRASS INTO MAP FIRST
landscapetable3:
  defw dograss                 ;  0 plains
  defw dograssflowers          ;  1 prairie
  defw dograssflowers          ;  2 forest
  defw dograssflowers          ;  3 knights
  defw dograsswestern          ;  4 western cowboy - cow skulls
  defw dograss                 ;  5 sea
  defw dograss                 ;  6 tropical
  defw dograss                 ;  7 winter
  defw dograssflowers          ;  8 autumn
  defw dograsswesternnoflowers ;  9 western indian - cactus
  defw dograss                 ; 10 forest mountain
  defw dograsswestern          ; 11 mountain western indian - skulls
  defw dograsswesternnoflowers ; 12 western tropical - cactus
  defw dograsswesternnoflowers ; 13 western tropical mountain - cactus

; DEFAULT FOREST TREES INDEXED BY ZONE ID
tabletrees:
  defw s_bigtree     ;  0 PLAINS
  defw s_bigtree     ;  1 PRAIRIE
  defw s_pinetree    ;  2 FOREST
  defw s_palm        ;  3 KNIGHTS
  defw s_bigcactus   ;  4 WESTERN COWBOY
  defw s_bigtree     ;  5 SEA
  defw s_palm        ;  6 TROPICAL
  defw s_bigtree     ;  7 WINTER
  defw s_bigtreebare ;  8 AUTUMN
  defw s_bigcactus   ;  9 WESTERN INDIAN
  defw s_pinetree    ; 10 FOREST MOUNTAIN
  defw s_bigcactus   ; 11 WESTERN MOUNT INDIAN
  defw s_pinetree    ; 12 WESTERN TROPICAL
  defw s_pinetree    ; 13 WESTERN TROPICAL MOUNTAIN

; RANDOM CHARACTERS BY ZONE
randcharbyzone:
  defw loadrandomforestsprite   ;  0 PLAINS
  defw loadrandomforestsprite   ;  1 PRAIRIE
  defw loadrandomloghousesprite ;  2 FOREST
  defw loadrandomknightsprite   ;  3 KNIGHTS
  defw loadrandomforestsprite   ;  4 WESTERN COWBOY
  defw loadrandomindiansprite   ;  5 SEA
  defw loadrandomindiansprite   ;  6 TROPICAL
  defw loadrandomforestsprite   ;  7 WINTER
  defw loadrandomforestsprite   ;  8 AUTUMN
  defw loadrandomindiansprite   ;  9 WESTERN INDIAN
  defw loadrandomloghousesprite ; 10 FOREST MOUNTAIN
  defw loadrandomindiansprite   ; 11 WESTERN MOUNTAIN
  defw loadrandomindiansprite   ; 12 WESTERN TROPICAL
  defw loadrandomindiansprite   ; 13 WESTERN TROPICAL MOUNTAIN